package trask.app.life;

import java.awt.*;
import java.awt.event.*;
import java.io.*;
import java.net.*;

public class LifeApplication extends Frame
{
  LifeApplet lifeApplet;
  URLDialog loadImageDialog = new URLDialog(this, "LoadImage");
  FileDialog loadPictureDialog = new FileDialog(this, "Load Picture");
  FileDialog savePictureDialog = new FileDialog(this, "Save Picture", FileDialog.SAVE);

  boolean applet = false;

  public LifeApplication()
  {
    MenuBar mbar = new MenuBar();
    setMenuBar(mbar);

    Menu fileMenu = new Menu("File");

    MenuItem resizePictureMI = new MenuItem("Resize Picture");
    resizePictureMI.addActionListener(new ActionListener() {
      public void actionPerformed(ActionEvent event) {
        lifeApplet.lifeDimensionControls.setVisible(true);
      }});
    fileMenu.add(resizePictureMI);

    MenuItem loadImageMI = new MenuItem("Load Image");
    loadImageMI.addActionListener(new ActionListener() {
      public void actionPerformed(ActionEvent event) {
        loadImageDialog.setVisible(true);
        URL imageURL = loadImageDialog.getURL();
        if (imageURL != null) {
          lifeApplet.lifePicture.loadImage(imageURL, lifeApplet.lifeColorControl.getColors());
        }
      }});
    fileMenu.add(loadImageMI);

    MenuItem loadPictureMI = new  MenuItem("Load Picture");
    loadPictureMI.addActionListener(new ActionListener() {
      public void actionPerformed(ActionEvent event) {
        loadPictureDialog.setVisible(true);
        if (loadPictureDialog.getFile() != null) {
          File pictureFile = new File(loadPictureDialog.getDirectory(), loadPictureDialog.getFile());
          lifeApplet.lifePicture.loadPicture(pictureFile, lifeApplet.lifeColorControl.getColors());
        }
      }});
    fileMenu.add(loadPictureMI);

    MenuItem savePictureMI = new MenuItem("Save Picture");
    savePictureMI.addActionListener(new ActionListener() {
      public void actionPerformed(ActionEvent event) {
        savePictureDialog.setVisible(true);
        if (savePictureDialog.getFile() != null) {
          File pictureFile = new File(savePictureDialog.getDirectory(), savePictureDialog.getFile());
          lifeApplet.lifePicture.savePicture(pictureFile);
        }
      }});
    fileMenu.add(savePictureMI);

    MenuItem printMI = new MenuItem("Print");
    printMI.addActionListener(new ActionListener() {
      public void actionPerformed(ActionEvent event) {
        printApplication();
      }});
    fileMenu.add(printMI);

    fileMenu.add(new MenuItem("-"));

    MenuItem exitMI = new MenuItem("Exit");
    exitMI.addActionListener(new ActionListener() {
      public void actionPerformed(ActionEvent event) {
        exitApplication();
      }});
    fileMenu.add(exitMI);

    mbar.add(fileMenu);

    addWindowListener(new WindowAdapter() {
      public void windowClosing(WindowEvent event) {
        exitApplication();
      }});

    lifeApplet = new LifeApplet();
    setLayout(new BorderLayout());
    add(lifeApplet, "Center");
    lifeApplet.init();
    pack();
  }

  public void exitApplication()
  {
    setVisible(false);
    if (!applet) {
      System.exit(0);
    }
  }

  public void printApplication()
  {
    PrintJob pjob = getToolkit().getPrintJob(this, "Printing Life", null);

    if (pjob != null) {
      Graphics pg = pjob.getGraphics();
      if (pg != null) {

        int w = lifeApplet.lifePicture.getSize().width;
        int h = lifeApplet.lifePicture.getSize().height;

        Graphics pg2 = pg.create(40, 40, w, h);
        lifeApplet.lifePicture.print(pg2);

        pg2 = pg.create(40 + w, 40, 1000, 1000);
        lifeApplet.calculationDisplay.print(pg2);

        pg2 = pg.create(40, 40 + h, 1000, 1000);
        lifeApplet.calculationDisplay.calculationDetails.print(pg2);

        pg.dispose(); // flush page
      }
      pjob.end();
    }
  }

  public Insets getInsets()
  {
    return new Insets(50, 10, 30, 10);
  }

  public static void main(String[] args) throws Exception
  {
    LifeApplication lifeApplication = new LifeApplication();
    lifeApplication.setVisible(true);
	}

}
