package trask.app.life;

import java.awt.*;
import java.awt.event.*;
import java.util.*;

public class LifeDimensionControls extends Frame
{
  private Dimension pictureDim = new Dimension(0,0);
  private Dimension elementDim = new Dimension(0,0);

  private TextField pictureWidthTF;
  private TextField pictureHeightTF;
  private TextField elementWidthTF;
  private TextField elementHeightTF;

  private Button okButton;
  private Button cancelButton;

  private Vector dimensionChangeListeners = new Vector();

  public LifeDimensionControls()
  {
    super("Dimension Controls");

    pictureWidthTF = new TextField(6);
    pictureHeightTF = new TextField(6);
    elementWidthTF = new TextField(6);
    elementHeightTF = new TextField(6);

    okButton = new Button("OK");
    okButton.addActionListener(new ActionListener() {
      public void actionPerformed(ActionEvent event) {
        if (applyChanges()) {
          setVisible(false);
          fireDimensionChangeEvent();
        }
      }});

    cancelButton = new Button("Cancel");
    cancelButton.addActionListener(new ActionListener() {
      public void actionPerformed(ActionEvent event) {
        cancelChanges();
        setVisible(false);
      }});

    addWindowListener(new WindowAdapter() {
      public void windowClosing(WindowEvent event) {
        cancelChanges();
        setVisible(false);
      }});

    setLayout(new GridBagLayout());
    GridBagConstraints constraints = new GridBagConstraints();

    // Picture Dimension Controls

    constraints.gridx = 1;
    constraints.gridy = 0;
    constraints.gridwidth = 1;
    constraints.anchor = GridBagConstraints.WEST;
    add(new Label("Picture"), constraints);

    constraints.gridx = 0;
    constraints.gridy = 1;
    constraints.gridwidth = 1;
    constraints.anchor = GridBagConstraints.EAST;
    add(new Label("Width (cols)"), constraints);

    constraints.gridx = 0;
    constraints.gridy = 2;
    constraints.gridwidth = 1;
    constraints.anchor = GridBagConstraints.EAST;
    add(new Label("Height (rows)"), constraints);

    constraints.gridx = 1;
    constraints.gridy = 1;
    constraints.gridwidth = 1;
    constraints.anchor = GridBagConstraints.WEST;
    add(pictureWidthTF, constraints);

    constraints.gridx = 1;
    constraints.gridy = 2;
    constraints.gridwidth = 1;
    constraints.anchor = GridBagConstraints.WEST;
    add(pictureHeightTF, constraints);

    // Element Dimension Controls

    constraints.gridx = 3;
    constraints.gridy = 0;
    constraints.gridwidth = 1;
    constraints.anchor = GridBagConstraints.WEST;
    add(new Label("Elements"), constraints);

    constraints.gridx = 2;
    constraints.gridy = 1;
    constraints.gridwidth = 1;
    constraints.anchor = GridBagConstraints.EAST;
    constraints.insets = new Insets(0,20,0,0);
    add(new Label("Width (pixels)"), constraints);
    constraints.insets = new Insets(0,0,0,0);

    constraints.gridx = 2;
    constraints.gridy = 2;
    constraints.gridwidth = 1;
    constraints.anchor = GridBagConstraints.EAST;
    add(new Label("Height (pixels)"), constraints);

    constraints.gridx = 3;
    constraints.gridy = 1;
    constraints.gridwidth = 1;
    constraints.anchor = GridBagConstraints.WEST;
    add(elementWidthTF, constraints);

    constraints.gridx = 3;
    constraints.gridy = 2;
    constraints.gridwidth = 1;
    constraints.anchor = GridBagConstraints.WEST;
    add(elementHeightTF, constraints);

    // OK and Cancel buttons

    constraints.gridx = 1;
    constraints.gridy = 3;
    constraints.gridwidth = 1;
    constraints.anchor = GridBagConstraints.SOUTH;
    constraints.insets = new Insets(20,0,10,0);
    constraints.ipadx = 20;
    add(okButton, constraints);
    constraints.ipadx = 0;

    constraints.gridx = 2;
    constraints.gridy = 3;
    constraints.gridwidth = 1;
    constraints.anchor = GridBagConstraints.SOUTH;
    add(cancelButton, constraints);
    constraints.insets = new Insets(0,0,0,0);

    pack();
    setLocation(200, 150);
  }

  // get and set methods

  public Dimension getPictureDim()
  {
    return new Dimension(pictureDim);
  }

  public void setPictureDim(Dimension picDim)
  {
    pictureDim = new Dimension(picDim);
    pictureWidthTF.setText("" + pictureDim.width);
    pictureHeightTF.setText("" + pictureDim.height);
  }

  public Dimension getElementDim()
  {
    return new Dimension(elementDim);
  }

  public void setElementDim(Dimension eltDim)
  {
    elementDim = new Dimension(eltDim);
    elementWidthTF.setText("" + elementDim.width);
    elementHeightTF.setText("" + elementDim.height);
  }

  public boolean applyChanges()
  {
    int pictureWidthTemp;
    int pictureHeightTemp;
    int elementWidthTemp;
    int elementHeightTemp;

    try {
      pictureWidthTemp = Integer.parseInt(pictureWidthTF.getText());
    }
    catch (NumberFormatException exc) {
      pictureWidthTF.requestFocus();
      pictureWidthTF.selectAll();
      return false;
    }
    try {
      pictureHeightTemp = Integer.parseInt(pictureHeightTF.getText());
    }
    catch (NumberFormatException exc) {
      pictureHeightTF.requestFocus();
      pictureHeightTF.selectAll();
      return false;
    }
    try {
      elementWidthTemp = Integer.parseInt(elementWidthTF.getText());
    }
    catch (NumberFormatException exc) {
      elementWidthTF.requestFocus();
      elementWidthTF.selectAll();
      return false;
    }
    try {
      elementHeightTemp = Integer.parseInt(elementHeightTF.getText());
    }
    catch (NumberFormatException exc) {
      elementHeightTF.requestFocus();
      elementHeightTF.selectAll();
      return false;
    }

    pictureDim.width = pictureWidthTemp;
    pictureDim.height = pictureHeightTemp;
    elementDim.width = elementWidthTemp;
    elementDim.height = elementHeightTemp;
    return true;
  }

  public void cancelChanges()
  {
    pictureWidthTF.setText("" + pictureDim.width);
    pictureHeightTF.setText("" + pictureDim.height);
    elementWidthTF.setText("" + elementDim.width);
    elementHeightTF.setText("" + elementDim.height);
  }

  // DimensionChangeEvent methods

  public void fireDimensionChangeEvent()
  {
    DimensionChangeEvent event = new DimensionChangeEvent(this,
                                          new Dimension(pictureDim),
                                          new Dimension(elementDim));

    for (Enumeration e = dimensionChangeListeners.elements(); e.hasMoreElements(); ) {
      DimensionChangeListener l = (DimensionChangeListener) e.nextElement();
      l.dimensionChanged(event);
    }
  }

  public synchronized void addDimensionChangeListener(DimensionChangeListener l)
  {
    dimensionChangeListeners.addElement(l);
  }

  public synchronized void removeDimensionChangeListener(DimensionChangeListener l)
  {
    dimensionChangeListeners.removeElement(l);
  }

  public Insets getInsets()
  {
    return new Insets(40, 20, 40, 20);
  }

}

